import { DynamicModule, Global, Inject, MiddlewareConsumer, Module, NestModule, RequestMethod } from '@nestjs/common';
import { RouteInfo } from '@nestjs/common/interfaces';
import { config } from 'dotenv';
import morgan from 'morgan';
import Transport from 'winston-transport';

import { LoggerNestAdapter } from './logger.nest.adapter';
import { LoggerService } from './logger.service';
import { Logger, LoggerBaseKey, LoggerKey, LoggerModuleOptions } from './logger.types';
import ConsoleTransport from './winston/transports/console.transport';
import FileTransport from './winston/transports/file.transport';
import SlackTransport from './winston/transports/slack.transport';
import { Levels } from './winston/transports/slack.transport.types';
import WinstonLogger, { WinstonLoggerTransportsKey } from './winston/winston.logger';

const defaultLevels: Levels = {
	debug: {
		send: false,
	},
	info: {
		send: false,
	},
	warn: {
		send: false,
	},
	error: {
		send: true,
	},
	emergency: {
		send: true,
	},
	fatal: {
		send: true,
	},
};

config();
@Global()
@Module({})
export class LoggerModule implements NestModule {
	static options: LoggerModuleOptions;

	public constructor(@Inject(LoggerKey) private logger: Logger) {}

	public configure(consumer: MiddlewareConsumer): void {
		consumer
			.apply(
				morgan('combined', {
					stream: {
						write: (message: string) => {
							const env = (process.env.NODE_ENV as string) ?? 'production';
							if (env.toLowerCase() === 'local') {
								this.logger.debug(message, 'RequestLogger');
							}
						},
					},
				})
			)
			.exclude(...(LoggerModule.options.morgan?.exclude || []))
			.forRoutes('*');
	}

	public static forRoot(options: LoggerModuleOptions): DynamicModule {
		LoggerModule.options = options;

		const levels = { ...defaultLevels, ...options.slack.levels };

		return {
			module: LoggerModule,
			imports: [],
			controllers: [],
			providers: [
				{
					provide: LoggerBaseKey,
					useClass: WinstonLogger,
				},
				{
					provide: LoggerKey,
					useClass: LoggerService,
				},
				{
					provide: LoggerNestAdapter,
					useFactory: (logger: Logger) => new LoggerNestAdapter(logger),
					inject: [LoggerKey],
				},

				{
					provide: WinstonLoggerTransportsKey,
					useFactory: () => {
						const transports: Transport[] = [];

						transports.push(ConsoleTransport.create(options.appName));

						transports.push(FileTransport.create());

						if (options.slack.enabled) {
							transports.push(
								SlackTransport.create({
									nodeEnv: options.nodeEnv,
									webhookUrl: options.slack.webhook?.url ?? '',
									appName: options.appName,
									levels,
								})
							);
						}

						return transports;
					},
				},
			],
			exports: [LoggerKey, LoggerNestAdapter],
		};
	}
}
