import { inspect } from 'util';
import winston, { format } from 'winston';
import { Console } from 'winston/lib/winston/transports';

const colors = {
	bold: (text: unknown) => `\x1B[1m${text}\x1B[0m`,
	red: (text: unknown) => `\x1B[31m${text}\x1B[39m`,
	green: (text: unknown) => `\x1B[32m${text}\x1B[39m`,
	yellow: (text: unknown) => `\x1B[33m${text}\x1B[39m`,
	blue: (text: unknown) => `\x1B[34m${text}\x1B[39m`,
	magenta: (text: unknown) => `\x1B[35m${text}\x1B[39m`,
	cyan: (text: unknown) => `\x1B[36m${text}\x1B[39m`,
	pink: (text: unknown) => `\x1B[38;5;206m${text}\x1B[39m`,
};

const levelColors = {
	debug: colors.blue,
	info: colors.green,
	warn: colors.yellow,
	error: colors.red,
	fatal: colors.magenta,
	emergency: colors.pink,
};

const customFormat = (appName: string, colorize = true) => {
	return format.printf((info: winston.Logform.TransformableInfo) => {
		const { timestamp, data, sourceClass, level, message, ms, trace_id } = info;
		let { context, stack } = info;

		if (context === undefined) {
			context = sourceClass;
		}
		if (context?.sourceClass !== undefined) {
			context = context.sourceClass;
		}

		if (stack === undefined) {
			stack = data;
		}

		if (stack?.sourceClass?.length === 0) {
			stack = undefined;
		}

		const color = colorize ? levelColors[level as keyof typeof levelColors] || ((text: unknown) => text) : (text: unknown) => text;
		const yellow = colorize ? colors.yellow : (text: unknown) => text;
		const formattedData = stack !== undefined ? inspect(stack, { colors: true, depth: null }) : undefined;

		return (
			(appName !== undefined ? `${color(`[${appName}]`)} ` : '') +
			(timestamp !== undefined ? `${color(timestamp)} ` : '') +
			(level !== undefined ? `${yellow(`[${level.toUpperCase()}]`)} ` : '') +
			(trace_id !== undefined ? `${yellow(`[TraceId][${trace_id}]`)} ` : '') +
			(context !== undefined ? `${yellow(`[${context}] `)}` : '') +
			`${color(message)}` +
			(formattedData !== undefined && formattedData !== '{}' ? ` - ${formattedData}` : '') +
			(ms !== undefined ? ` ${yellow(ms)}` : '')
		);
	});
};

const getFormat = (formatters: winston.Logform.Format[] = []) =>
	format.combine(format.timestamp({ format: 'YYYY-MM-DD HH:mm:ss' }), format.errors({ stack: true }), format.ms(), ...formatters);

export default class ConsoleTransport {
	public static create(appName: string) {
		return new Console({
			format: getFormat([customFormat(appName, true)]),
		});
	}
}
